# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" 
[ Awaken 自动化框架 ]

---
描述
    - Awaken 是一个功能自动化测试框架, 将支持 WEB、API、APP 等平台。
    - 关键字驱动技术是该框架的一个重要思想, 它将测试用例设计与自动化实现开发分离开来;
    - 测试人员不需要具备很深的脚本语言知识, 应用该框架自实现的一套精简的关键字语法即可快速编写用例逻辑。

"""


# --------------------------------------------------------------------------
class AwakenDetails:
    """ 
    [ Awaken详情结构体 ] 
    
    """
    Name     = 'awaken2'
    """ 项目名称 """
    Version  = '0.2.4'
    """ 项目版本 """
    Desc     = 'Awaken 是一个功能自动化测试框架, 将支持 WEB、API、APP 等平台。'
    """ 项目简介 """
    Url      = 'https://gitee.com/catcat7/awaken2'
    """ 项目URL """
    Author   = 'chenjiancheng'
    """ 项目作者 """
    Email    = 'quinn.7@foxmail.com'
    """ 作者邮箱 """
    Keywords = ['awaken2', 'python', 'testing', 'automation', 'testautomation']
    """ 项目关键字 """
