# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 加解密相关函数 ]

"""
import hashlib


# --------------------------------------------------------------------------
def encrypt_md5(string: str) -> str:
    """
    [ MD5加密 ]

    ---
    参数
    - string : { str } - 需要加密的字符串。

    ---
    返回
    - str : 加密后的字符串。

    """
    md5hash = hashlib.md5(string.encode('UTF-8'))
    return md5hash.hexdigest()
